@@ -1,4 +1,4 @@
-### Copyright (C) 2002-2004 Stephen Kennedy <stevek@gnome.org>
+### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
 
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -14,11 +14,7 @@
 ### along with this program; if not, write to the Free Software
 ### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from __future__ import generators
-
 import paths
-import diffutil
-import errno
 import gnomeglade
 import gobject
 import gtk
@@ -26,10 +22,11 @@ import gtk.keysyms
 import math
 import misc
 import os
+from gettext import gettext as _
+from gettext import ngettext
 import shutil
 import melddoc
 import tree
-import filecmp
 import re
 import stat
 import time
@@ -150,43 +147,17 @@ class EmblemCellRenderer(gtk.GenericCellRenderer):
 
     def on_render(self, window, widget, background_area, cell_area, expose_area, flags):
         r = self.renderer
-        r.set_property("pixbuf", self.pixbuf)
+        r.props.pixbuf = self.pixbuf
         r.render(window, widget, background_area, cell_area, expose_area, flags)
-        r.set_property("pixbuf", self.emblem)
+        r.props.pixbuf = self.emblem
         r.render(window, widget, background_area, cell_area, expose_area, flags)
 
     def on_get_size(self, widget, cell_area):
         if not hasattr(self, "size"):
             r = self.renderer
-            r.set_property("pixbuf", self.pixbuf)
+            r.props.pixbuf = self.pixbuf
             self.size = r.get_size(widget, cell_area)
         return self.size
-gobject.type_register(EmblemCellRenderer)
-
-################################################################################
-#
-# DirDiffMenu
-#
-################################################################################
-class DirDiffMenu(gnomeglade.Component):
-    def __init__(self, app):
-        gladefile = paths.share_dir("glade2/dirdiff.glade")
-        gnomeglade.Component.__init__(self, gladefile, "popup")
-        self.parent = app
-    def popup_in_pane( self, pane ):
-        self.copy_left.set_sensitive( pane > 0 )
-        self.copy_right.set_sensitive( pane+1 < self.parent.num_panes )
-        self.widget.popup( None, None, None, 3, gtk.get_current_event_time() )
-    def on_popup_compare_activate(self, menuitem):
-        self.parent.launch_comparisons_on_selected()
-    def on_popup_copy_left_activate(self, menuitem):
-        self.parent.on_button_copy_left_clicked( None )
-    def on_popup_copy_right_activate(self, menuitem):
-        self.parent.on_button_copy_right_clicked( None )
-    def on_popup_delete_activate(self, menuitem):
-        self.parent.on_button_delete_clicked( None )
-    def on_popup_edit_activate(self, menuitem):
-        self.parent.on_button_edit_clicked( None )
 
 ################################################################################
 #
@@ -213,10 +184,52 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def __init__(self, prefs, num_panes):
         melddoc.MeldDoc.__init__(self, prefs)
         gnomeglade.Component.__init__(self, paths.share_dir("glade2/dirdiff.glade"), "dirdiff")
+
+        actions = (
+            ("DirCompare",   gtk.STOCK_DIALOG_INFO,  _("_Compare"), None, _("Compare selected"), self.on_button_diff_clicked),
+            # FIXME: the glade files were inconsistent: GO_BACK vs GOTO_FIRST, "Left" vs "Copy To Left"
+            ("DirCopyLeft",  gtk.STOCK_GO_BACK,      _("Left"),     None, _("Copy To Left"), self.on_button_copy_left_clicked),
+            # FIXME: the glade files were inconsistent: GO_FORWARD vs GOTO_LAST, "Right" vs "Copy To Right"
+            ("DirCopyRight", gtk.STOCK_GO_FORWARD,   _("Right"),    None, _("Copy To Right"), self.on_button_copy_right_clicked),
+            ("DirDelete",    gtk.STOCK_DELETE,        None,         None, _("Delete selected"), self.on_button_delete_clicked),
+            ("Hide",         gtk.STOCK_NO,           _("Hide..."),  None, _("Hide selected"), self.on_filter_hide_current_clicked),
+
+            ("DirOpen",      gtk.STOCK_OPEN,          None,         None, _("Open selected"), self.on_button_open_clicked),
+        )
+
+        toggleactions = (
+            ("IgnoreCase",   gtk.STOCK_ITALIC,  _("Case"),     None, _("Ignore case of entries"), self.on_button_ignore_case_toggled, False),
+            ("ShowSame",     gtk.STOCK_APPLY,   _("Same"),     None, _("Show identical"), self.on_filter_state_normal_toggled, True),
+            ("ShowNew",      gtk.STOCK_ADD,     _("New"),      None, _("Show new"), self.on_filter_state_new_toggled, True),
+            ("ShowModified", gtk.STOCK_REMOVE,  _("Modified"), None, _("Show modified"), self.on_filter_state_modified_toggled, True),
+        )
+        ui_file = paths.share_dir("glade2/dirdiff-ui.xml")
+        self.ui = gtk.UIManager()
+        self.actiongroup = gtk.ActionGroup('DirdiffToolbarActions')
+        self.actiongroup.set_translation_domain("meld")
+        self.actiongroup.add_actions(actions)
+        self.actiongroup.add_toggle_actions(toggleactions)
+        self.ui.insert_action_group(self.actiongroup, 0)
+        self.ui.add_ui_from_file(ui_file)
+        self.create_name_filters()
+        for button in ("DirCompare", "DirCopyLeft", "DirCopyRight",
+                       "DirDelete", "Hide", "IgnoreCase", "ShowSame",
+                       "ShowNew", "ShowModified"):
+            self.actiongroup.get_action(button).props.is_important = True
+        self.toolbar = self.ui.get_widget('/DirdiffToolbar')
+        self.dirdiff.pack_start(self.toolbar, False, True, 0)
         self.toolbar.set_style( self.prefs.get_toolbar_style() )
-        self._map_widgets_into_lists( ["treeview", "fileentry", "diffmap", "scrolledwindow", "linkmap"] )
-        self.popup_menu = DirDiffMenu(self)
+        self.map_widgets_into_lists( ["treeview", "fileentry", "diffmap", "scrolledwindow", "linkmap"] )
+        self.popup_menu = self.ui.get_widget('/DirdiffPopup')
+        self.popup_menu.connect("deactivate", self.on_popup_deactivate_event)
         self.set_num_panes(num_panes)
+        self.focus_in_events = []
+        self.focus_out_events = []
+        for treeview in self.treeview:
+            handler_id = treeview.connect("focus-in-event", self.on_treeview_focus_in_event)
+            self.focus_in_events.append(handler_id)
+            handler_id = treeview.connect("focus-out-event", self.on_treeview_focus_out_event)
+            self.focus_out_events.append(handler_id)
         self.on_treeview_focus_out_event(None, None)
         self.treeview_focussed = None
 
@@ -239,7 +252,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             tree.STATE_MODIFIED,
             tree.STATE_NEW,
         ]
-        self.create_name_filters()
         self.update_regexes()
 
     def update_regexes(self):
@@ -248,7 +260,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if r.active:
                 try:
                     self.regexes.append( re.compile(r.value+"(?m)") )
-                except re.error, e:
+                except re.error:
                     misc.run_dialog(
                         text=_("Error converting pattern '%s' to regular expression") % r.value )
 
@@ -264,24 +276,20 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 continue
             try:
                 cregex = re.compile(regex)
-            except re.error, e:
-                misc.run_dialog( _("Error converting pattern '%s' to regular expression") % pattern, self )
+            except re.error:
+                misc.run_dialog( _("Error converting pattern '%s' to regular expression") % f.value, self )
             else:
                 func = lambda x, r=cregex : r.match(x) == None
                 self.name_filters_available.append( TypeFilter(f.name, f.active, func) )
         self.name_filters = []
-        tips = gtk.Tooltips()
-        for i,f in misc.enumerate(self.name_filters_available):
-            icon = gtk.Image()
-            icon.set_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_LARGE_TOOLBAR)
-            toggle = gtk.ToggleToolButton()
-            toggle.set_property("label",f.label)
-            toggle.set_icon_widget(icon)
-            toggle.connect("toggled", lambda b,i=i : self._update_name_filter(b,i) )
-            toggle.set_active(f.active)
-            self.toolbar.insert(toggle, -1)
-            toggle.show_all()
-            toggle.set_tooltip(tips, _("Hide %s") % f.label )
+        merge_id = self.ui.new_merge_id()
+        for i,f in enumerate(self.name_filters_available):
+            name = "Hide%d" % i
+            entry = (name, gtk.STOCK_FIND, f.label, None, _("Hide %s") % f.label, lambda b,i=i : self._update_name_filter(b,i), f.active)
+            self.actiongroup.add_toggle_actions([entry])
+            if f.active:
+                self.name_filters.append(self.name_filters_available[i])
+            self.ui.add_ui(merge_id, "/DirdiffToolbar/FilterButtons", name, name, gtk.UI_MANAGER_TOOLITEM, False)
 
     def on_preference_changed(self, key, value):
         if key == "toolbar_style":
@@ -316,20 +324,20 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def file_deleted(self, path, pane):
         # is file still extant in other pane?
-        iter = self.model.get_iter(path)
-        files = self.model.value_paths(iter)
-        is_present = [ os.path.exists( file ) for file in files ]
+        it = self.model.get_iter(path)
+        files = self.model.value_paths(it)
+        is_present = [ os.path.exists(f) for f in files ]
         if 1 in is_present:
-            self._update_item_state(iter)
+            self._update_item_state(it)
         else: # nope its gone
-            self.model.remove(iter)
+            self.model.remove(it)
         self._update_diffmaps()
 
     def file_created(self, path, pane):
-        iter = self.model.get_iter(path)
-        while iter and self.model.get_path(iter) != (0,):
-            self._update_item_state( iter )
-            iter = self.model.iter_parent(iter)
+        it = self.model.get_iter(path)
+        while it and self.model.get_path(it) != (0,):
+            self._update_item_state( it )
+            it = self.model.iter_parent(it)
         self._update_diffmaps()
 
     def on_fileentry_activate(self, entry):
@@ -340,9 +348,10 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.set_num_panes(len(locations))
         locations = [os.path.abspath(l or ".") for l in locations]
         self.model.clear()
-        for pane, loc in misc.enumerate(locations):
+        for pane, loc in enumerate(locations):
             self.fileentry[pane].set_filename(loc)
         child = self.model.add_entries(None, locations)
+        self.treeview0.grab_focus()
         self._update_item_state(child)
         self.recompute_label()
         self.scheduler.remove_all_tasks()
@@ -351,28 +360,29 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def recursively_update( self, path ):
         """Recursively update from tree path 'path'.
         """
-        iter = self.model.get_iter( path )
-        child = self.model.iter_children( iter )
+        it = self.model.get_iter( path )
+        child = self.model.iter_children( it )
         while child:
             self.model.remove(child)
-            child = self.model.iter_children( iter )
-        self._update_item_state(iter)
+            child = self.model.iter_children( it )
+        self._update_item_state(it)
         self.scheduler.add_task( self._search_recursively_iter( path ).next )
 
     def _search_recursively_iter(self, rootpath):
-        self.filter_hide_current.set_sensitive(False)
-        yield _("[%s] Scanning") % self.label_text
+        self.actiongroup.get_action("Hide").set_sensitive(False)
+        yield _("[%s] Scanning %s") % (self.label_text, "")
         prefixlen = 1 + len( self.model.value_path( self.model.get_iter(rootpath), 0 ) )
         symlinks_followed = {} # only follow symlinks once
         todo = [ rootpath ]
+        expanded = {}
         while len(todo):
             todo.sort() # depth first
             path = todo.pop(0)
-            iter = self.model.get_iter( path )
-            roots = self.model.value_paths( iter )
+            it = self.model.get_iter( path )
+            roots = self.model.value_paths( it )
             yield _("[%s] Scanning %s") % (self.label_text, roots[0][prefixlen:])
             differences = [0]
-            if not self.button_ignore_case.get_active():
+            if not self.actiongroup.get_action("IgnoreCase").get_active():
                 class accum(object):
                     def __init__(self, parent, roots):
                         self.items = []
@@ -422,40 +432,41 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                         return [ fixup(k, self.items[k]) for k in keys ]
             accumdirs = accum(self, roots)
             accumfiles = accum(self, roots)
-            for pane, root in misc.enumerate(roots):
+            for pane, root in enumerate(roots):
                 if os.path.isdir( root ):
                     try:
                         entries = os.listdir( root )
                     except OSError, err:
-                        self.model.add_error( iter, err.strerror, pane )
+                        self.model.add_error( it, err.strerror, pane )
                         differences = [1]
                     else:
                         for f in self.name_filters:
                             entries = filter(f.filter, entries)
+                        files = []
+                        dirs = []
                         for e in entries:
                             s = os.lstat( join(root,e) )
-                            files = []
-                            dirs = []
-                            if stat.S_ISREG(s.st_mode):
+                            if stat.S_ISLNK(s.st_mode):
+                                if not self.prefs.ignore_symlinks:
+                                    key = (s.st_dev, s.st_ino)
+                                    if symlinks_followed.get( key, 0 ) == 0:
+                                        symlinks_followed[key] = 1
+                                        try:
+                                            s = os.stat( join(root,e) )
+                                        except OSError, err:
+                                            print "ignoring dangling symlink", e
+                                            pass
+                                        else:
+                                            if stat.S_ISREG(s.st_mode):
+                                                files.append(e)
+                                            elif stat.S_ISDIR(s.st_mode):
+                                                dirs.append(e)
+                            elif stat.S_ISREG(s.st_mode):
                                 files.append(e)
                             elif stat.S_ISDIR(s.st_mode):
                                 dirs.append(e)
-                            elif stat.S_ISLNK(s.st_mode):
-                                key = (s.st_dev, s.st_ino)
-                                if symlinks_followed.get( key, 0 ) == 0:
-                                    symlinks_followed[key] = 1
-                                    try:
-                                        s = os.stat( join(root,e) )
-                                    except OSError, err:
-                                        print "ignoring dangling symlink", e
-                                        pass
-                                    else:
-                                        if stat.S_ISREG(s.st_mode):
-                                            files.append(e)
-                                        elif stat.S_ISDIR(s.st_mode):
-                                            dirs.append(e)
-                            accumfiles.add( pane, files )
-                            accumdirs.add( pane, dirs )
+                        accumfiles.add( pane, files )
+                        accumdirs.add( pane, dirs )
 
             alldirs = accumdirs.get()
             allfiles = self._filter_on_state( roots, accumfiles.get() )
@@ -463,30 +474,34 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             # then directories and files
             if len(alldirs) + len(allfiles) != 0:
                 def add_entry(names):
-                    child = self.model.add_entries( iter, [join(r,n) for r,n in zip(roots, names) ] )
+                    child = self.model.add_entries( it, [join(r,n) for r,n in zip(roots, names) ] )
                     differences[0] |= self._update_item_state(child)
                     return child
                 map(lambda x : todo.append( self.model.get_path(add_entry(x))), alldirs )
                 map(add_entry, allfiles)
             else: # directory is empty, add a placeholder
-                self.model.add_empty(iter)
+                self.model.add_empty(it)
             if differences[0]:
-                start = path[:]
-                while len(start) and not self.treeview[0].row_expanded(start):
-                    start = start[:-1]
-                level = len(start)
-                while level < len(path):
-                    level += 1
-                    self.treeview[0].expand_row( path[:level], 0)
+                 expanded[path] = False
+        for path in sorted(expanded.keys()):
+            start = path[:]
+            while len(start) and not expanded.get(start,False):
+                start = start[:-1]
+            level = len(start)
+            while level < len(path):
+                level += 1
+                cur = path[:level]
+                expanded[cur] = True
+                self.treeview[0].expand_row( cur, 0)
         yield _("[%s] Done") % self.label_text
-        self.filter_hide_current.set_sensitive(True)
+        self.actiongroup.get_action("Hide").set_sensitive(True)
 
-    def launch_comparison(self, iter, pane, force=1):
-        """Launch comparison at 'iter'. 
+    def launch_comparison(self, it, pane, force=1):
+        """Launch comparison at 'it'. 
            If it is a file we launch a diff.
            If it is a folder we recursively open diffs for each non equal file.
         """
-        paths = filter(os.path.exists, self.model.value_paths(iter))
+        paths = filter(os.path.exists, self.model.value_paths(it))
         self.emit("create-diff", paths)
 
     def launch_comparisons_on_selected(self):
@@ -509,18 +524,18 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             paths.reverse()
             model = self.model
             for path in paths: #filter(lambda x: x.name!=None, sel):
-                iter = model.get_iter(path)
-                name = model.value_path(iter, src_pane)
+                it = model.get_iter(path)
+                name = model.value_path(it, src_pane)
                 if name == None:
                     continue
-                src = model.value_path(iter, src_pane)
-                dst = model.value_path(iter, dst_pane)
+                src = model.value_path(it, src_pane)
+                dst = model.value_path(it, dst_pane)
                 try:
                     if os.path.isfile(src):
                         dstdir = os.path.dirname( dst )
                         if not os.path.exists( dstdir ):
                             os.makedirs( dstdir )
-                        shutil.copy( src, dstdir )
+                        shutil.copy2( src, dstdir )
                         self.file_created( path, dst_pane)
                     elif os.path.isdir(src):
                         if os.path.exists(dst):
@@ -542,8 +557,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             paths = self._get_selected_paths(pane)
             paths.reverse()
             for path in paths:
-                iter = self.model.get_iter(path)
-                name = self.model.value_path(iter, pane)
+                it = self.model.get_iter(path)
+                name = self.model.value_path(it, pane)
                 try:
                     if os.path.isfile(name):
                         os.remove(name)
@@ -578,9 +593,9 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     if abs(int(deltat)) < 5 * units:
                         return msg(int(deltat)) % int(deltat)
                     deltat /= units
-            file = self.model.value_path( self.model.get_iter(paths[0]), pane )
+            fname = self.model.value_path( self.model.get_iter(paths[0]), pane )
             try:
-                stat = os.stat(file)
+                stat = os.stat(fname)
             except OSError:
                 self.emit("status-changed", "" )
             else:
@@ -613,12 +628,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         return event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right) #handled
 
     def on_treeview_row_activated(self, view, path, column):
-        iter = self.model.get_iter(path)
+        it = self.model.get_iter(path)
         files = []
         for i in range(self.num_panes):
-            file = self.model.value_path( iter, i )
-            if os.path.exists(file):
-                files.append(file)
+            fname = self.model.value_path( it, i )
+            if os.path.exists(fname):
+                files.append(fname)
             else:
                 files.append(None)
         if files.count(None) != self.num_panes:
@@ -635,7 +650,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 else:
                     view.expand_row(path,0)
 
-    def on_treeview_row_expanded(self, view, iter, path):
+    def on_treeview_row_expanded(self, view, it, path):
         self._do_to_others(view, self.treeview, "expand_row", (path,0) )
         self._update_diffmaps()
 
@@ -643,18 +658,23 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._do_to_others(view, self.treeview, "collapse_row", (path,) )
         self._update_diffmaps()
 
+    def on_popup_deactivate_event(self, popup):
+        for (treeview, inid, outid) in zip(self.treeview, self.focus_in_events, self.focus_out_events):
+            treeview.handler_unblock(inid)
+            treeview.handler_unblock(outid)
+
     def on_treeview_focus_in_event(self, tree, event):
         self.treeview_focussed = tree
         pane = self.treeview.index(tree)
         if pane > 0:
-            self.button_copy_left.set_sensitive(1)
+            self.actiongroup.get_action("DirCopyLeft").set_sensitive(True)
         if pane+1 < self.num_panes:
-            self.button_copy_right.set_sensitive(1)
-        self.button_delete.set_sensitive(1)
+            self.actiongroup.get_action("DirCopyRight").set_sensitive(True)
+        self.actiongroup.get_action("DirDelete").set_sensitive(True)
     def on_treeview_focus_out_event(self, tree, event):
-        self.button_copy_left.set_sensitive(0)
-        self.button_copy_right.set_sensitive(0)
-        self.button_delete.set_sensitive(0)
+        self.actiongroup.get_action("DirCopyLeft").set_sensitive(False)
+        self.actiongroup.get_action("DirCopyRight").set_sensitive(False)
+        self.actiongroup.get_action("DirDelete").set_sensitive(False)
         #
         # Toolbar handlers
         #
@@ -668,12 +688,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.copy_selected(1)
     def on_button_delete_clicked(self, button):
         self.delete_selected()
-    def on_button_edit_clicked(self, button):
+    def on_button_open_clicked(self, button):
         pane = self._get_focused_pane()
         if pane != None:
             m = self.model
             files = [ m.value_path( m.get_iter(p), pane ) for p in self._get_selected_paths(pane) ]
-            self._edit_files( [f for f in files if os.path.isfile(f)] )
+            self._open_files(files)
 
     def on_button_ignore_case_toggled(self, button):
         self.refresh()
@@ -718,7 +738,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         assert pane != None
         selected_paths = []
         self.treeview[pane].get_selection().selected_foreach(
-            lambda store, path, iter: selected_paths.append( path ) )
+            lambda store, path, it: selected_paths.append( path ) )
         return selected_paths
 
         #
@@ -749,17 +769,17 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 ret.append( files )
         return ret
 
-    def _update_item_state(self, iter):
-        """Update the state of the item at 'iter'
+    def _update_item_state(self, it):
+        """Update the state of the item at 'it'
         """
-        files = self.model.value_paths(iter)
+        files = self.model.value_paths(it)
         def mtime(f):
             try:
                 return os.stat(f).st_mtime
             except OSError:
                 return 0
         # find the newest file, checking also that they differ
-        mod_times = [ mtime( file ) for file in files[:self.num_panes] ]
+        mod_times = [ mtime(f) for f in files[:self.num_panes] ]
         newest_index = mod_times.index( max(mod_times) )
         if mod_times.count( max(mod_times) ) == len(mod_times):
             newest_index = -1 # all same
@@ -779,22 +799,30 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if mod_times[j]:
                 isdir = os.path.isdir( files[j] )
                 if all_same == 1:
-                    self.model.set_state(iter, j,  tree.STATE_NORMAL, isdir)
+                    self.model.set_state(it, j,  tree.STATE_NORMAL, isdir)
                     different = 0
                 elif all_same == 2:
-                    self.model.set_state(iter, j,  tree.STATE_NOCHANGE, isdir)
+                    self.model.set_state(it, j,  tree.STATE_NOCHANGE, isdir)
                     different = 0
                 elif all_present_same:
-                    self.model.set_state(iter, j,  tree.STATE_NEW, isdir)
+                    self.model.set_state(it, j,  tree.STATE_NEW, isdir)
                 else:
-                    self.model.set_state(iter, j,  tree.STATE_MODIFIED, isdir)
-                self.model.set_value(iter,
+                    self.model.set_state(it, j,  tree.STATE_MODIFIED, isdir)
+                self.model.set_value(it,
                     self.model.column_index(COL_EMBLEM, j),
                     j == newest_index and pixbuf_newer or None)
             else:
-                self.model.set_state(iter, j,  tree.STATE_MISSING)
+                self.model.set_state(it, j,  tree.STATE_MISSING)
         return different
 
+    def popup_in_pane(self, pane):
+        for (treeview, inid, outid) in zip(self.treeview, self.focus_in_events, self.focus_out_events):
+            treeview.handler_block(inid)
+            treeview.handler_block(outid)
+        self.actiongroup.get_action("DirCopyLeft").set_sensitive(pane > 0)
+        self.actiongroup.get_action("DirCopyRight").set_sensitive(pane+1 < self.num_panes)
+        self.popup_menu.popup(None, None, None, 3, gtk.get_current_event_time())
+
     def on_treeview_button_press_event(self, treeview, event):
         # unselect other panes
         for t in filter(lambda x:x!=treeview, self.treeview[:self.num_panes]):
@@ -809,8 +837,9 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 selected = self._get_selected_paths( self.treeview.index(treeview) )
                 if len(selected) <= 1 and event.state == 0:
                     treeview.set_cursor( path, col, 0)
-                self.popup_menu.popup_in_pane( self.treeview.index(treeview) )
+                self.popup_in_pane( self.treeview.index(treeview) )
             return event.state==0
+        return 0
 
     def set_num_panes(self, n):
         if n != self.num_panes and n in (1,2,3):
@@ -855,13 +884,13 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             todo = [root]
             model = self.model
             while len(todo):
-                iter = todo.pop(0)
-                #print model.value_path(iter, treeindex), model.get_state(iter, treeindex)
-                yield model.get_state(iter, treeindex)
-                path = model.get_path(iter)
+                it = todo.pop(0)
+                #print model.value_path(it, treeindex), model.get_state(it, treeindex)
+                yield model.get_state(it, treeindex)
+                path = model.get_path(it)
                 if treeview.row_expanded(path):
                     children = []
-                    child = model.iter_children(iter)
+                    child = model.iter_children(it)
                     while child:
                         children.append(child)
                         child = model.iter_next(child)
@@ -940,6 +969,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             upper = adj.upper - adj.page_size
             adj.set_value( max( min(upper, val), 0) )
             return 1
+        return 0
 
     def on_file_changed(self, changed_filename):
         """When a file has changed, try to find it in our tree
@@ -1000,4 +1030,6 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 goto_iter(it)
                 return
 
-gobject.type_register(DirDiff)
+    def on_reload_activate(self, *extra):
+        self.on_fileentry_activate(None)
+
