@@ -17,12 +17,14 @@
 # system
 import sys
 import os
+from gettext import gettext as _
 
 # gnome
 import gtk
 import gtk.glade
 import gobject
 import pango
+import gnome
 
 # project
 import paths
@@ -63,7 +65,7 @@ class NewDocDialog(gnomeglade.Component):
     def __init__(self, parentapp, type):
         self.parentapp = parentapp
         gnomeglade.Component.__init__(self, paths.share_dir("glade2/meldapp.glade"), "newdialog")
-        self._map_widgets_into_lists( ("fileentry", "direntry", "vcentry", "three_way_compare", "tablabel") )
+        self.map_widgets_into_lists( ("fileentry", "direntry", "vcentry", "three_way_compare", "tablabel") )
         self.entrylists = self.fileentry, self.direntry, self.vcentry
         self.widget.set_transient_for(parentapp.widget)
         cur_page = type // 2
@@ -113,7 +115,7 @@ class ListWidget(gnomeglade.Component):
         def addTextCol(label, colnum, expand=0):
             model = view.get_model()
             rentext = gtk.CellRendererText()
-            rentext.set_property("editable", 1)
+            rentext.props.editable = 1
             def change_text(ren, row, text):
                 it = model.get_iter( (int(row),))
                 model.set_value( it, colnum, text)
@@ -227,23 +229,29 @@ class PreferencesDialog(gnomeglade.Component):
                 self.model.append( (label,) )
         self.prefs = parentapp.prefs
         # editor
-        self._map_widgets_into_lists( ["editor_command"] )
+        self.map_widgets_into_lists( ["editor_command"] )
         if self.prefs.use_custom_font:
             self.radiobutton_custom_font.set_active(1)
         else:
             self.radiobutton_gnome_font.set_active(1)
         self.fontpicker.set_font_name( self.prefs.custom_font )
         self.spinbutton_tabsize.set_value( self.prefs.tab_size )
+        if sourceview_available:
+            self.checkbutton_spaces_instead_of_tabs.set_active( self.prefs.spaces_instead_of_tabs )
+            self.checkbutton_show_line_numbers.set_active( self.prefs.show_line_numbers )
+            self.checkbutton_use_syntax_highlighting.set_active( self.prefs.use_syntax_highlighting )
+        else:
+            self.checkbutton_spaces_instead_of_tabs.set_sensitive(False)
+            self.checkbutton_show_line_numbers.set_sensitive(False)
+            self.checkbutton_use_syntax_highlighting.set_sensitive(False)
         self.option_wrap_lines.set_history( self.prefs.edit_wrap_lines )
         self.checkbutton_supply_newline.set_active( self.prefs.supply_newline )
-        self.checkbutton_show_line_numbers.set_active( self.prefs.show_line_numbers )
-        self.checkbutton_use_syntax_highlighting.set_active( self.prefs.use_syntax_highlighting )
         self.editor_command[ self.editor_radio_values.get(self.prefs.edit_command_type, "internal") ].set_active(1)
         self.gnome_default_editor_label.set_text( "(%s)" % " ".join(self.prefs.get_gnome_editor_command([])) )
         self.custom_edit_command_entry.set_text( " ".join(self.prefs.get_custom_editor_command([])) )
         # display
-        self._map_widgets_into_lists( ["draw_style"] )
-        self._map_widgets_into_lists( ["toolbar_style"] )
+        self.map_widgets_into_lists( ["draw_style"] )
+        self.map_widgets_into_lists( ["toolbar_style"] )
         self.draw_style[self.prefs.draw_style].set_active(1)
         self.toolbar_style[self.prefs.toolbar_style].set_active(1)
         # file filters
@@ -258,7 +266,7 @@ class PreferencesDialog(gnomeglade.Component):
         self.checkbutton_ignore_blank_lines.set_active( self.prefs.ignore_blank_lines )
         # encoding
         self.entry_text_codecs.set_text( self.prefs.text_codecs )
-        self._map_widgets_into_lists( ["save_encoding"] )
+        self.map_widgets_into_lists( ["save_encoding"] )
         self.save_encoding[self.prefs.save_encoding].set_active(1)
         self.treeview.set_cursor(0)
     #
@@ -279,6 +287,10 @@ class PreferencesDialog(gnomeglade.Component):
             self.prefs.use_custom_font = custom
     def on_spinbutton_tabsize_changed(self, spin):
         self.prefs.tab_size = int(spin.get_value())
+    def on_checkbutton_spaces_instead_of_tabs_toggled(self, check):
+        self.prefs.spaces_instead_of_tabs = check.get_active()
+        if check.get_active() and not sourceview_available:
+            misc.run_dialog(_("Spaces instead of tab is only available if you have gnome-python-desktop installed.") )
     def on_option_wrap_lines_changed(self, option):
         self.prefs.edit_wrap_lines = option.get_history()
     def on_checkbutton_supply_newline_toggled(self, check):
@@ -323,8 +335,8 @@ class PreferencesDialog(gnomeglade.Component):
             self.prefs.save_encoding = self.save_encoding.index(radio)
     def on_response(self, dialog, arg):
         if arg==gtk.RESPONSE_CLOSE:
-            self.prefs.text_codecs = self.entry_text_codecs.get_property("text")
-            self.prefs.edit_command_custom = self.custom_edit_command_entry.get_property("text")
+            self.prefs.text_codecs = self.entry_text_codecs.props.text
+            self.prefs.edit_command_custom = self.custom_edit_command_entry.props.text
         self.widget.destroy()
 
 ################################################################################
@@ -378,7 +390,8 @@ class NotebookLabel(gtk.HBox):
         button.set_relief(gtk.RELIEF_NONE)
         button.set_focus_on_click(False)
         image = gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
-        image.set_tooltip_text(_("Close tab"))
+        if gtk.pygtk_version >= (2,12,0):
+            image.set_tooltip_text(_("Close tab"))
         button.add(image)
         button.set_name("meld-tab-close-button")
         button.set_size_request(w + 2, h + 2)
@@ -388,20 +401,33 @@ class NotebookLabel(gtk.HBox):
         icon.set_from_file( paths.share_dir("glade2/pixmaps/%s" % iconname) )
         icon.set_from_pixbuf(icon.get_pixbuf().scale_simple(16, 16, 2)) #TODO stock image
 
+        label_box = gtk.EventBox()
+        label_box.add_events(gtk.gdk.BUTTON_PRESS_MASK)
+        label_box.props.visible_window = False
+        label_box.connect("button-press-event", self.on_label_clicked)
+        label_box.add(label)
+
         self.pack_start(icon, expand=False)
-        self.pack_start(label)
+        self.pack_start(label_box)
         self.pack_start(button, expand=False)
-        self.set_tooltip_text(text)
+        if gtk.pygtk_version >= (2,12,0):
+            self.set_tooltip_text(text)
         self.show_all()
 
         self.__label = label
+        self.__onclose = onclose
+
+    def on_label_clicked(self, box, event):
+        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 2:
+            self.__onclose(None)
 
     def get_label_text(self):
         return self.__label.get_text()
 
     def set_label_text(self, text):
         self.__label.set_text(text)
-        self.set_tooltip_text(text)
+        if gtk.pygtk_version >= (2,12,0):
+            self.set_tooltip_text(text)
 
 ################################################################################
 #
@@ -415,6 +441,7 @@ class MeldPreferences(prefs.Preferences):
         "use_custom_font": prefs.Value(prefs.BOOL,0),
         "custom_font": prefs.Value(prefs.STRING,"monospace, 14"),
         "tab_size": prefs.Value(prefs.INT, 4),
+        "spaces_instead_of_tabs": prefs.Value(prefs.BOOL, False),
         "show_line_numbers": prefs.Value(prefs.BOOL, 0),
         "use_syntax_highlighting": prefs.Value(prefs.BOOL, 0),
         "edit_wrap_lines" : prefs.Value(prefs.INT, 0),
@@ -507,7 +534,7 @@ class MeldPreferences(prefs.Preferences):
 # MeldApp
 #
 ################################################################################
-class MeldApp(gnomeglade.GnomeApp):
+class MeldApp(gnomeglade.Component):
 
     #
     # init
@@ -522,12 +549,61 @@ class MeldApp(gnomeglade.GnomeApp):
                      "}\n"
                      "widget \"*.meld-tab-close-button\" style \"meld-tab-close-button-style\"")
         gladefile = paths.share_dir("glade2/meldapp.glade")
-        gnomeglade.GnomeApp.__init__(self, "meld", version, gladefile, "meldapp")
-        self._map_widgets_into_lists( "settings_drawstyle".split() )
+        self.program = gnome.program_init("meld", version)
+        gnomeglade.Component.__init__(self, gladefile, "meldapp")
+
+        actions = (
+            ("FileMenu", None, "_File"),
+            ("New",     gtk.STOCK_NEW,      _("_New..."), "<control>N", _("Start a new comparison"), self.on_menu_file_new_activate),
+            ("Save",    gtk.STOCK_SAVE,     None, None, _("Save the current file"), self.on_menu_save_activate),
+            ("SaveAs",  gtk.STOCK_SAVE_AS,  None, "<control><shift>S", "Save the current file with a different name", self.on_menu_save_as_activate),
+            ("Close",   gtk.STOCK_CLOSE,    None, None, _("Close the current file"), self.on_menu_close_activate),
+            ("Quit",    gtk.STOCK_QUIT,     None, None, _("Quit the program"), self.on_menu_quit_activate),
+
+            ("EditMenu", None, "_Edit"),
+            ("Undo",    gtk.STOCK_UNDO,     None, "<control>Z", _("Undo the last action"), self.on_menu_undo_activate),
+            ("Redo",    gtk.STOCK_REDO,     None, "<control><shift>Z", _("Redo the last undone action"), self.on_menu_redo_activate),
+            ("Cut",     gtk.STOCK_CUT,      None, None, _("Cut the selection"), self.on_menu_cut_activate),
+            ("Copy",    gtk.STOCK_COPY,     None, None, _("Copy the selection"), self.on_menu_copy_activate),
+            ("Paste",   gtk.STOCK_PASTE,    None, None, _("Paste the clipboard"), self.on_menu_paste_activate),
+            ("Find",    gtk.STOCK_FIND,     None, None, _("Search for text"), self.on_menu_find_activate),
+            ("FindNext", None,              _("Find Ne_xt"), "<control>G", _("Search forwards for the same text"), self.on_menu_find_next_activate),
+            ("Down",    gtk.STOCK_GO_DOWN,  None, "<control>D", _("Go to the next difference"), self.on_menu_edit_down_activate),
+            ("Up",      gtk.STOCK_GO_UP,    None, "<control>E", _("Go to the previous difference"), self.on_menu_edit_up_activate),
+            ("Preferences", gtk.STOCK_PREFERENCES, None, None, _("Configure the application"), self.on_menu_preferences_activate),
+
+            ("ViewMenu", None, "_View"),
+            ("Stop",    gtk.STOCK_STOP,     None, "Escape", _("Stop the current action"), self.on_toolbar_stop_clicked),
+            ("Refresh", gtk.STOCK_REFRESH,  None, "<control>R", _("Refresh the view"), self.on_menu_refresh_activate),
+            ("Reload",  gtk.STOCK_REFRESH,  _("Reload"), "<control><shift>R", _("Reload the comparison"), self.on_menu_reload_activate),
+
+            ("HelpMenu", None, "_Help"),
+            ("Help",        gtk.STOCK_HELP,  _("_Contents"), "F1", _("Open the Meld manual"), self.on_menu_users_manual_activate),
+            ("BugReport",   gtk.STOCK_DIALOG_WARNING, _("Report _Bug"), None, _("Report a bug in Meld"), self.on_menu_help_bug_activate),
+            ("MailingList", None,            _("Mailing _List"), None, _("Go to the Meld mailing list"), self.on_menu_about_activate),
+            ("About",       gtk.STOCK_ABOUT, None, None, _("About this program"), self.on_menu_about_activate),
+
+            ("Magic",       gtk.STOCK_YES,   None,     None, None, self.on_menu_magic_activate),
+        )
+        ui_file = paths.share_dir("glade2/meldapp-ui.xml")
+        self.actiongroup = gtk.ActionGroup('MainActions')
+        self.actiongroup.set_translation_domain("meld")
+        self.actiongroup.add_actions(actions)
+        self.ui = gtk.UIManager()
+        self.ui.insert_action_group(self.actiongroup, 0)
+        self.ui.add_ui_from_file(ui_file)
+        for menuitem in ("New", "Save", "Undo", "Redo"):
+            self.actiongroup.get_action(menuitem).props.is_important = True
+        self.widget.add_accel_group(self.ui.get_accel_group())
+        self.menubar = self.ui.get_widget('/Menubar')
+        self.toolbar = self.ui.get_widget('/Toolbar')
+        self.appvbox.pack_start(self.menubar, expand=False)
+        self.appvbox.pack_start(self.toolbar, expand=False)
+        self.map_widgets_into_lists( "settings_drawstyle".split() )
         self.statusbar = MeldStatusBar(self.task_progress, self.task_status, self.doc_status)
         self.prefs = MeldPreferences()
         if not developer:#hide magic testing button
-            self.toolbar_magic.hide()
+            self.ui.get_widget("/Toolbar/Magic").hide()
         elif 1:
             def showPrefs(): PreferencesDialog(self)
             gobject.idle_add(showPrefs)
@@ -537,6 +613,7 @@ class MeldApp(gnomeglade.GnomeApp):
         self.scheduler = task.LifoScheduler()
         self.scheduler.connect("runnable", self.on_scheduler_runnable )
         self.widget.set_default_size(self.prefs.window_size_x, self.prefs.window_size_y)
+        self.ui.ensure_update()
         self.widget.show()
 
     def on_idle(self):
@@ -551,14 +628,12 @@ class MeldApp(gnomeglade.GnomeApp):
         else:
             self.statusbar.task_progress.set_fraction(0)
         if self.scheduler.tasks_pending():
-            self.toolbar_stop.set_sensitive(1)
-            self.menu_view_stop.set_sensitive(1)
+            self.actiongroup.get_action("Stop").set_sensitive(True)
             return 1
         else:
             self.statusbar.set_task_status("")
             self.idle_hooked = 0
-            self.toolbar_stop.set_sensitive(0)
-            self.menu_view_stop.set_sensitive(0)
+            self.actiongroup.get_action("Stop").set_sensitive(False)
             return 0
 
     def on_scheduler_runnable(self, sched):
@@ -579,8 +654,8 @@ class MeldApp(gnomeglade.GnomeApp):
     def on_switch_page(self, notebook, page, which):
         newdoc = notebook.get_nth_page(which).get_data("pyobject")
         newseq = newdoc.undosequence
-        self.button_undo.set_sensitive(newseq.can_undo())
-        self.button_redo.set_sensitive(newseq.can_redo())
+        self.actiongroup.get_action("Undo").set_sensitive(newseq.can_undo())
+        self.actiongroup.get_action("Redo").set_sensitive(newseq.can_redo())
         nbl = self.notebook.get_tab_label( newdoc.widget )
         self.widget.set_title(nbl.get_label_text() + " - Meld")
         self.statusbar.set_doc_status("")
@@ -594,10 +669,10 @@ class MeldApp(gnomeglade.GnomeApp):
         self.notebook.child_set_property(component.widget, "menu-label", text)
 
     def on_can_undo(self, undosequence, can):
-        self.button_undo.set_sensitive(can)
+        self.actiongroup.get_action("Undo").set_sensitive(can)
 
     def on_can_redo(self, undosequence, can):
-        self.button_redo.set_sensitive(can)
+        self.actiongroup.get_action("Redo").set_sensitive(can)
 
     def on_size_allocate(self, window, rect):
         self.prefs.window_size_x = rect.width
@@ -636,7 +711,7 @@ class MeldApp(gnomeglade.GnomeApp):
                     break
         for c in self.notebook.get_children():
             c.get_data("pyobject").on_quit_event()
-        self.quit()
+        gtk.main_quit()
         return gtk.RESPONSE_CLOSE
 
     #
@@ -685,18 +760,18 @@ class MeldApp(gnomeglade.GnomeApp):
     # Toolbar and menu items (help)
     #
     def on_menu_meld_home_page_activate(self, button):
-        gnomeglade.url_show("http://meld.sourceforge.net")
+        gnome.url_show("http://meld.sourceforge.net")
 
     def on_menu_help_bug_activate(self, button):
-        gnomeglade.url_show("http://bugzilla.gnome.org/buglist.cgi?product=meld")
+        gnome.url_show("http://bugzilla.gnome.org/buglist.cgi?product=meld")
 
     def on_menu_users_manual_activate(self, button):
-        gnomeglade.url_show("ghelp:///"+os.path.abspath(paths.help_dir("C/meld.xml") ))
+        gnome.url_show("ghelp:///"+os.path.abspath(paths.help_dir("C/meld.xml") ))
 
     def on_menu_about_activate(self, *extra):
         about = gtk.glade.XML(paths.share_dir("glade2/meldapp.glade"),"about").get_widget("about")
-        about.set_property("name", "Meld")
-        about.set_property("version", version)
+        about.props.name = "Meld"
+        about.props.version = version
         about.show()
 
     #
@@ -743,14 +818,17 @@ class MeldApp(gnomeglade.GnomeApp):
         self.scheduler.add_scheduler(page.scheduler)
         page.connect("label-changed", self.on_notebook_label_changed)
         page.connect("file-changed", self.on_file_changed)
-        page.connect("create-diff", lambda obj,arg: self.append_filediff(arg) )
+        page.connect("create-diff", lambda obj,arg: self.append_diff(arg) )
         page.connect("status-changed", lambda junk,arg: self.statusbar.set_doc_status(arg) )
 
-    def append_dirdiff(self, dirs):
+    def append_dirdiff(self, dirs, auto_compare=False):
         assert len(dirs) in (1,2,3)
         doc = dirdiff.DirDiff(self.prefs, len(dirs))
         self._append_page(doc, "tree-folder-normal.png")
         doc.set_locations(dirs)
+        # FIXME: This doesn't work, as dirdiff behaves differently to vcview
+        if auto_compare:
+            doc.on_button_diff_clicked(None)
         return doc
 
     def append_filediff(self, files):
@@ -764,7 +842,7 @@ class MeldApp(gnomeglade.GnomeApp):
         doc.set_files(files)
         return doc
 
-    def append_diff(self, paths):
+    def append_diff(self, paths, auto_compare=False):
         aredirs = [ os.path.isdir(p) for p in paths ]
         arefiles = [ os.path.isfile(p) for p in paths ]
         if (1 in aredirs) and (1 in arefiles):
@@ -772,16 +850,18 @@ class MeldApp(gnomeglade.GnomeApp):
                     parent = self,
                     buttonstype = gtk.BUTTONS_OK)
         elif 1 in aredirs:
-            return self.append_dirdiff(paths)
+            return self.append_dirdiff(paths, auto_compare)
         else:
             return self.append_filediff(paths)
 
-    def append_vcview(self, locations):
+    def append_vcview(self, locations, auto_compare=False):
         assert len(locations) in (1,)
         location = locations[0]
         doc = vcview.VcView(self.prefs)
         self._append_page(doc, "vc-icon.png")
         doc.set_location(location)
+        if auto_compare:
+            doc.on_button_diff_clicked(None)
         return doc
 
     #
@@ -837,6 +917,7 @@ def main():
     sys.stdout = Unbuffered(sys.stdout)
 
     parser = optparse.OptionParser(
+    option_class=misc.MeldOption,
     usage="""
     %prog                       Start with no windows open
     %prog <dir>                 Start with VC browser in 'dir'
@@ -846,15 +927,26 @@ def main():
     description="""Meld is a file and directory comparison tool.""",
     version="%prog "+version)
     parser.add_option("-L", "--label", action="append", default=[], help=_("Set label to use instead of file name"))
+    parser.add_option("-a", "--auto-compare", action="store_true", default=False, help=_("Automatically compare all differing files on startup"))
     parser.add_option("-u", "--unified", action="store_true", help=_("Ignored for compatibility"))
     parser.add_option("-c", "--context", action="store_true", help=_("Ignored for compatibility"))
     parser.add_option("-e", "--ed", action="store_true", help=_("Ignored for compatibility"))
     parser.add_option("-r", "--recursive", action="store_true", help=_("Ignored for compatibility"))
+    parser.add_option("", "--diff", action="diff_files", dest='diff',
+                      default=[],
+                      help=_("Creates a diff tab for up to 3 supplied files."))
     options, args = parser.parse_args()
 
+    icon_theme = gtk.icon_theme_get_default()
+    icon_theme.append_search_path(paths.share_dir("glade2/pixmaps/"))
     app = MeldApp()
     tab = None
 
+    for files in options.diff:
+        if len(files) not in (1, 2, 3):
+            app.usage(_("Invalid number of arguments supplied for --diff."))
+        app.append_diff(files)
+
     if len(args) == 0:
         pass
 
@@ -870,14 +962,14 @@ def main():
             doc.connect("create-diff", lambda obj,arg: app.append_diff(arg) )
             doc.run_diff([a])
         else:
-            tab = app.append_vcview( [a] )
+            tab = app.append_vcview([a], options.auto_compare)
                 
     elif len(args) in (2,3):
-        tab = app.append_diff(args)
+        tab = app.append_diff(args, options.auto_compare)
     else:
         app.usage( _("Wrong number of arguments (Got %i)") % len(args))
 
     if tab:
         tab.set_labels( options.label )
-    app.main()
+    gtk.main()
 
