@@ -22,6 +22,7 @@
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import os
+from gettext import gettext as _
 
 # ignored, new, normal, ignored changes,
 # error, placeholder, vc added
@@ -70,9 +71,6 @@ class Vc(object):
 
     PATCH_STRIP_NUM = 0
 
-    def __init__(self, location):
-        pass
-
     def commit_command(self, message):
         raise NotImplementedError()
     def diff_command(self):
@@ -100,6 +98,9 @@ class Vc(object):
     def uncache_inventory(self):
         pass
 
+    def listdir_filter(self, entries):
+        return [f for f in entries if f[0]!="." and f!="CVS"]
+
     def listdir(self, start):
         if start=="": start="."
         if start[-1] != "/": start+="/"
@@ -110,7 +111,7 @@ class Vc(object):
             entries.sort()
         except OSError:
             entries = []
-        for f in [f for f in entries if f[0]!="." and f!="CVS" and f!=".svn"]:
+        for f in self.listdir_filter(entries):
             fname = start + f
             lname = fname
             if os.path.isdir(fname):
